package com.gxzc.zen.api.sys.service.impl

import com.baomidou.mybatisplus.mapper.EntityWrapper
import com.baomidou.mybatisplus.service.impl.ServiceImpl
import com.gxzc.zen.api.sys.mapper.SysUserRoleMapper
import com.gxzc.zen.api.sys.model.SysRole
import com.gxzc.zen.api.sys.model.SysUserRole
import com.gxzc.zen.api.sys.service.ISysUserRoleService
import com.gxzc.zen.umps.constant.ZenHttpSession
import com.gxzc.zen.umps.util.ShiroRedisUtil
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

/**
 * <p>
 * 用户角色表 服务实现类
 * </p>
 *
 * @author NorthLan
 * @since 2018-02-06
 */
@Service
class SysUserRoleServiceImpl : ServiceImpl<SysUserRoleMapper, SysUserRole>(), ISysUserRoleService {
    override fun getUserRoleListByUserId(id: Long): MutableList<SysRole> {
        return baseMapper.selectUserRoleListByUserId(id)
    }

    override fun getUserRoleIdListByUserId(id: Long): List<Long> {
        val condition = SysUserRole().apply { this.userId = id }
        return baseMapper.selectList(EntityWrapper(condition)).map { it.roleId!! }
    }

    override fun getUserIdListByRoleId(id: Long): Set<Long> {
        val ret = baseMapper.selectList(EntityWrapper<SysUserRole>().eq("role_id", id)) ?: return setOf()
        return ret.map {
            it.userId!!
        }.toSet()
    }

    @Transactional
    override fun deleteByUserId(userId: Long) {
        baseMapper.physicalDelete(EntityWrapper<SysUserRole>()
                .eq("user_id", userId))
    }

    @Transactional
    override fun updateByUserId(userId: Long, roleIds: List<Long>) {
        // 先查询userId 所有的roleIds
        val tempRoleIds = getUserRoleIdListByUserId(userId)
        val deleteIds = tempRoleIds.subtract(roleIds)
        val insertIds = roleIds.subtract(tempRoleIds)

        var eff = false
        if (deleteIds.isNotEmpty()) {
            baseMapper.physicalDelete(EntityWrapper<SysUserRole>().eq("user_id", userId).`in`("role_id", deleteIds))
            eff = true
        }

        if (insertIds.isNotEmpty()) {
            val entityList = mutableListOf<SysUserRole>()
            insertIds.forEach {
                entityList.add(SysUserRole().apply {
                    this.roleId = it
                    this.userId = userId
                })
            }
            this.insertBatch(entityList)
            eff = true
        }

        if (eff) {
            // 更新一下
            ShiroRedisUtil.removeAllSessionsAttributeKey(ZenHttpSession.SESSION_KEY_USER_ROLES)
        }
    }
}
