package com.gxzc.zen.web.sys.controller

import com.gxzc.zen.api.sys.model.SysRole
import com.gxzc.zen.api.sys.service.ISysRoleService
import com.gxzc.zen.common.base.BaseController
import com.gxzc.zen.common.config.response.annotation.ZenResponseFilter
import com.gxzc.zen.common.dto.ResponseDto
import com.gxzc.zen.common.exception.ZenException
import com.gxzc.zen.common.exception.ZenExceptionEnum
import com.gxzc.zen.common.util.PaginationUtil
import io.swagger.annotations.ApiOperation
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*

/**
 * 角色 控制器
 * @author NorthLan
 * @date 2018/3/24
 * @url https://noahlan.com
 */
@RestController
@RequestMapping("role")
class RoleController : BaseController() {
    companion object {
        private val logger = LoggerFactory.getLogger(RoleController::class.java)
    }

    @Autowired
    private lateinit var roleService: ISysRoleService

    @ApiOperation("查询role列表")
    @GetMapping("list")
    @ZenResponseFilter(type = SysRole::class, filter = ["createBy", "createTime", "updateBy", "updateTime"])
    fun list(@RequestParam(required = false) keyword: String?,
             @RequestParam(required = false) searchOption: Int?,
             @RequestParam(required = false) enable: Boolean?): ResponseEntity<*> {
        val result: Any = if (PaginationUtil.paginable(getRequest())) {
            roleService.getRoleListPage(keyword, searchOption, PaginationUtil.getRequestPage(getRequest())!!, enable)
        } else {
            roleService.getRoleList(keyword, searchOption, enable)
        }
        return ResponseEntity.ok(ResponseDto().apply { data = result })
    }

    @ApiOperation("查询指定id的role")
    @GetMapping("{id}")
    @ZenResponseFilter(type = SysRole::class, filter = ["createBy", "updateBy", "createTime", "updateTime"])
    fun getById(@PathVariable id: Long): ResponseEntity<*> {
        return ResponseEntity.ok(ResponseDto().apply {
            data = roleService.selectById(id)
        })
    }

    @PostMapping
    @ZenResponseFilter(type = SysRole::class, filter = ["createBy", "updateBy", "createTime", "updateTime"])
    fun insertRole(@RequestBody data: SysRole): ResponseEntity<*> {
        if (!roleService.insert(data)) {
            throw ZenException(ZenExceptionEnum.BIZ_INSERT_ERROR)
        }
        return ResponseEntity.ok(ResponseDto().apply {
            this.data = data
        }) // 201
    }

    @PutMapping
    @ZenResponseFilter(type = SysRole::class, filter = ["createBy", "updateBy", "createTime", "updateTime"])
    fun putDic(@RequestBody data: SysRole): ResponseEntity<*> {
        roleService.updateLogicById(data)
        return ResponseEntity.ok(ResponseDto().apply {
            this.data = data
        })
    }

    @DeleteMapping("{id}")
    fun deleteDic(@PathVariable id: Long): ResponseEntity<*> {
        // 物理删除数据
        roleService.physicalDeleteById(id)
        return ResponseEntity.ok(ResponseDto())
    }
}